@@ -1,6 +1,7 @@
 /******************************************************************************
  *                                                                            *
- * Copyright (C) 2006-2009 by Tor Andersson.                                  *
+ * Copyright (C) 2006-2009 by Tor Andersson, Jesse McGrew.                    *
+ * Copyright (C) 2010 by Ben Cressey, Chris Spiegel.                          *
  *                                                                            *
  * This file is part of Gargoyle.                                             *
  *                                                                            *
@@ -82,7 +83,7 @@ typedef struct window_graphics_s window_graphics_t;
 /* Some globals for gargoyle */
 
 #define TBLINELEN 300
-#define SCROLLBACK 1024
+#define SCROLLBACK 512
 #define HISTORYLEN 100
 
 #define GLI_SUBPIX 8
@@ -97,6 +98,7 @@ extern window_t *gli_focuswin;
 extern event_t *gli_curevent;
 
 extern int gli_force_redraw;
+extern int gli_more_focus;
 extern int gli_cellw;
 extern int gli_cellh;
 
@@ -123,6 +125,7 @@ extern int gli_cellh;
 
 typedef struct rect_s rect_t;
 typedef struct picture_s picture_t;
+typedef struct piclist_s piclist_t;
 typedef struct style_s style_t;
 typedef struct mask_s mask_t;
 
@@ -137,6 +140,15 @@ struct picture_s
     int refcount;
     int w, h;
     unsigned char *rgba;
+    unsigned long id;
+    int scaled;
+};
+
+struct piclist_s
+{
+    picture_t *picture;
+    picture_t *scaled;
+    struct piclist_s *next;
 };
 
 struct style_s
@@ -181,8 +193,10 @@ extern unsigned char gli_caret_save[3];
 extern unsigned char gli_more_save[3];
 extern unsigned char gli_link_save[3];
 
-extern int gli_override_fg;
-extern int gli_override_bg;
+extern int gli_override_fg_set;
+extern int gli_override_bg_set;
+extern int gli_override_fg_val;
+extern int gli_override_bg_val;
 extern int gli_override_reverse;
 
 extern int gli_link_style;
@@ -220,13 +234,17 @@ extern int gli_scroll_width;
 extern int gli_baseline;
 extern int gli_leading;
 
-enum { MONOR, MONOB, MONOI, MONOZ, PROPR, PROPB, PROPI, PROPZ };
+enum FACES { MONOR, MONOB, MONOI, MONOZ, PROPR, PROPB, PROPI, PROPZ };
+enum TYPES { MONOF, PROPF };
+enum STYLES { FONTR, FONTB, FONTI, FONTZ };
 
+extern char *gli_conf_propfont;
 extern char *gli_conf_propr;
 extern char *gli_conf_propb;
 extern char *gli_conf_propi;
 extern char *gli_conf_propz;
 
+extern char *gli_conf_monofont;
 extern char *gli_conf_monor;
 extern char *gli_conf_monob;
 extern char *gli_conf_monoi;
@@ -277,6 +295,7 @@ struct eventqueue_s
 eventqueue_t *gli_initialize_queue (void);
 void gli_queue_event(eventqueue_t *queue, event_t *event);
 event_t *gli_retrieve_event(eventqueue_t *queue);
+void gli_dispatch_event(event_t *event, int polled);
 
 #define MAGIC_WINDOW_NUM (9876)
 #define MAGIC_STREAM_NUM (8769)
@@ -335,12 +354,14 @@ struct glk_fileref_struct
 
 typedef struct attr_s
 {
-    unsigned bgcolor : 4;
-    unsigned fgcolor : 4;
+    unsigned fgset   : 1;
+    unsigned bgset   : 1;
+    unsigned fgcolor : 24;
+    unsigned bgcolor : 24;
     unsigned style   : 4;
     unsigned reverse : 1;
     unsigned hyper   : 4;
-    unsigned		 : 3;
+    unsigned         : 5;
 } attr_t;
 
 struct glk_window_struct
@@ -363,6 +384,9 @@ struct glk_window_struct
     int char_request_uni;
     int mouse_request;
     int hyper_request;
+    int more_request;
+    int scroll_request;
+    int image_loaded;
 
     attr_t attr;
     unsigned char bgcolor[3];
@@ -438,7 +462,8 @@ struct window_textbuffer_s
     int spaced;
     int dashed;
 
-    tbline_t lines[SCROLLBACK];	/* XXX make this dynamic */
+    tbline_t *lines;
+    int scrollback;
 
     int numchars;		/* number of chars in last line: lines[0] */
     glui32 *chars;		/* alias to lines[0].chars */
@@ -472,7 +497,7 @@ struct window_textbuffer_s
     style_t styles[style_NUMSTYLES];
 
     /* for copy selection */
-    glui32 copybuf[SCROLLBACK + SCROLLBACK * TBLINELEN];
+    glui32 *copybuf;
     int copypos;
 };
 
@@ -561,6 +586,7 @@ extern void win_textbuffer_cancel_line(window_t *win, event_t *ev);
 extern void win_textbuffer_click(window_textbuffer_t *dwin, int x, int y);
 extern void gcmd_buffer_accept_readchar(window_t *win, glui32 arg);
 extern void gcmd_buffer_accept_readline(window_t *win, glui32 arg);
+extern void gcmd_accept_scroll(window_t *win, glui32 arg);
 
 /* Declarations of library internal functions. */
 
@@ -581,8 +607,12 @@ extern void gli_windows_size_change(void);
 
 extern void gli_window_click(window_t *win, int x, int y);
 
-void gli_input_next_focus();
+void gli_redraw_rect(int x0, int y0, int x1, int y1);
+
 void gli_input_guess_focus();
+void gli_input_more_focus();
+void gli_input_next_focus();
+void gli_input_scroll_focus();
 void gli_input_handle_key(glui32 key);
 void gli_input_handle_click(int x, int y);
 void gli_event_store(glui32 type, window_t *win, glui32 val1, glui32 val2);
@@ -629,14 +659,22 @@ void winrepaint(int x0, int y0, int x1, int y1);
 void winabort(const char *fmt, ...);
 void winopenfile(char *prompt, char *buf, int buflen, char *filter);
 void winsavefile(char *prompt, char *buf, int buflen, char *filter);
+void winexit(void);
+void winclipstore(glui32 *text, int len);
+
+void fontreplace(char *font, int type);
+void fontload(void);
+void fontunload(void);
 
 int giblorb_is_resource_map();
 void giblorb_get_resource(glui32 usage, glui32 resnum, FILE **file, long *pos, long *len, glui32 *type);
 
 picture_t *gli_picture_load(unsigned long id);
-void gli_picture_keep(picture_t *pic);
-void gli_picture_drop(picture_t *pic);
+void gli_picture_store(picture_t *pic);
+picture_t *gli_picture_retrieve(unsigned long id, int scaled);
 picture_t *gli_picture_scale(picture_t *src, int destwidth, int destheight);
+void gli_piclist_increment(void);
+void gli_piclist_decrement(void);
 
 window_graphics_t *win_graphics_create(window_t *win);
 void win_graphics_destroy(window_graphics_t *cutwin);
@@ -673,37 +711,19 @@ glui32 gli_parse_utf8(unsigned char *buf, glui32 buflen, glui32 *out, glui32 out
 
 glui32 strlen_uni(glui32 *s);
 
+void gli_put_hyperlink(glui32 linkval, unsigned int x0, unsigned int y0, unsigned int x1, unsigned int y1);
+glui32 gli_get_hyperlink(unsigned int x, unsigned int y);
+void gli_clear_selection(void);
+int gli_check_selection(unsigned int x0, unsigned int y0, unsigned int x1, unsigned int y1);
+int gli_get_selection(unsigned int x0, unsigned int y0, unsigned int x1, unsigned int y1, unsigned int *rx0, unsigned int *rx1);
+void gli_clipboard_copy(glui32 *buf, int len);
+void gli_start_selection(int x, int y);
+void gli_resize_mask(unsigned int x, unsigned int y);
+void gli_move_selection(int x, int y);
+
 void attrset(attr_t *attr, glui32 style);
 void attrclear(attr_t *attr);
 int attrequal(attr_t *a1, attr_t *a2);
 unsigned char *attrfg(style_t *styles, attr_t *attr);
 unsigned char *attrbg(style_t *styles, attr_t *attr);
 int attrfont(style_t *styles, attr_t *attr);
-
-/* RGB color values for garglk_set_zcolors(), from Z-machine standard 1.1 draft 9 */
-static unsigned char zcolor_rgb[][3] = {
-    { 0, 0, 0 },        /* zcolor_Black */
-    { 239, 0, 0 },      /* zcolor_Red */
-    { 0, 214, 0 },      /* zcolor_Green */
-    { 239, 239, 0 },    /* zcolor_Yellow */
-    { 0, 107, 181 },    /* zcolor_Blue */
-    { 255, 0, 255 },    /* zcolor_Magenta */
-    { 0, 239, 239 },    /* zcolor_Cyan */
-    { 255, 255, 255 },  /* zcolor_White */
-    { 181, 181, 181 },  /* zcolor_LightGrey */
-    { 140, 140, 140 },  /* zcolor_MediumGrey */
-    { 90, 90, 90 },     /* zcolor_DarkGrey */
-};
-static unsigned char zbright_rgb[][3] = {
-    { 48, 48, 48 },        /* zbright_Black */
-    { 255, 48, 48 },      /* zbright_Red */
-    { 48, 255, 48 },      /* zbright_Green */
-    { 255, 255, 48 },    /* zbright_Yellow */
-    { 0, 155, 229 },    /* zbright_Blue */
-    { 255, 48, 255 },    /* zbright_Magenta */
-    { 48, 255, 255 },    /* zbright_Cyan */
-    { 255, 255, 255 },  /* zbright_White */
-    { 229, 229, 229 },  /* zbright_LightGrey */
-    { 188, 188, 188 },  /* zbright_MediumGrey */
-    { 138, 138, 138 },     /* zbright_DarkGrey */
-};
