@@ -1,6 +1,7 @@
 /******************************************************************************
  *                                                                            *
- * Copyright (C) 2006-2009 by Tor Andersson.                                  *
+ * Copyright (C) 2006-2009 by Tor Andersson, Jesse McGrew.                    *
+ * Copyright (C) 2010 by Ben Cressey, Chris Spiegel, Jrg Walter.             *
  *                                                                            *
  * This file is part of Gargoyle.                                             *
  *                                                                            *
@@ -115,7 +116,7 @@ void win_textgrid_redraw(window_t *win)
     int i, a, b, k, o;
     glui32 link;
     int font;
-    char *fgcolor, *bgcolor;
+    unsigned char *fgcolor, *bgcolor;
 
     x0 = win->bbox.x0;
     x1 = win->bbox.x1;
@@ -270,8 +271,10 @@ void win_textgrid_clear(window_t *win)
     window_textgrid_t *dwin = win->data;
     int k, j;
 
-    win->attr.fgcolor = gli_override_fg;
-    win->attr.bgcolor = gli_override_bg;
+    win->attr.fgset = gli_override_fg_set;
+    win->attr.bgset = gli_override_bg_set;
+    win->attr.fgcolor = gli_override_fg_set ? gli_override_fg_val : 0;
+    win->attr.bgcolor = gli_override_bg_set ? gli_override_bg_val : 0;
     win->attr.reverse = FALSE;
 
     for (k = 0; k < dwin->height; k++)
@@ -308,11 +311,13 @@ void win_textgrid_click(window_textgrid_t *dwin, int sx, int sy)
     int x = sx - win->bbox.x0;
     int y = sy - win->bbox.y0;
 
-    if (win->line_request || win->char_request || win->line_request_uni || win->char_request_uni)
-    gli_focuswin = win;
+    if (win->line_request || win->char_request
+        || win->line_request_uni || win->char_request_uni
+        || win->more_request || win->scroll_request)
+        gli_focuswin = win;
 
     if (win->mouse_request) {
-        gli_event_store(evtype_MouseInput, win, x, y);
+        gli_event_store(evtype_MouseInput, win, x/gli_cellw, y/gli_leading);
         win->mouse_request = FALSE;
         if (gli_conf_safeclicks)
             gli_forceclick = 1;
